#
#  Name:    MpCa.py
#
#  Purpose: MX Python interface to EPICS Channel Access.
#
#  Author:  William Lavender
#
#---------------------------------------------------------------------------
#
#  Copyright 2009 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

import Mp
import MpCaPrivate

DBE_VALUE = 1
DBE_ALARM = 2
DBE_LOG   = 4

MpCaPrivate.init()

#---------------------------------------------------------------------------

def pend_io( timeout ):
	MpCaPrivate.pend_event( timeout )

def pend_event( timeout ):
	MpCaPrivate.pend_event( timeout )

def poll():
	MpCaPrivate.pend_event( 1.0e-12 )

def flush_io():
	MpCaPrivate.flush_io()

#---------------------------------------------------------------------------

class Callback:
	def __init__( self, callback_mask, \
			callback_function, callback_arguments ):
		self.debug = False

		if self.debug:
			print "MpCa.Callback( %s, %s, %s ) invoked" \
		    % ( callback_mask, callback_function, callback_arguments )

		self.callback_mask = callback_mask
		self.callback_function = callback_function
		self.callback_arguments = callback_arguments

		if self.debug:
			print "MpCa.Callback() constructor complete for %s" \
				% self

	def __del__( self ):
		if self.debug:
			print "MpCa.Callback() destructor invoked for %s" % self
			
		MpCaPrivate.cb_delete_callback( self )

		if self.debug:
			print "MpCa.Callback() destructor complete for %s" \
				% self

#---------------------------------------------------------------------------

class PV:
	def __init__( self, pvname ):
		self.debug = False

		if self.debug:
			print "MpCa.PV( %s ) invoked" % pvname

		self.pvname = pvname
		self.handle = MpCaPrivate.pv_init( pvname )

	def caget( self, timeout=-1.0, remote=True ):
		result = MpCaPrivate.pv_caget( self.handle, timeout, remote )

		if self.debug:
			print "MpCa.PV.caget() received %s from '%s'" \
				% (str(result), self.pvname)

		return result

	def caput( self, args, timeout=-1.0, wait=True ):
		if self.debug:
			print "MpCa.PV.caput() sending %s to '%s'" \
				% (str(args), self.pvname)

		MpCaPrivate.pv_caput( self.handle, timeout, wait, args )

	def get_local( self ):
		result = MpCaPrivate.pv_caget( self.handle, -1.0, False )

		if self.debug:
			print "MpCa.PV.get_local() has %s for '%s'" \
				% (str(result), self.pvname)

		return result

	def add_callback( self, \
			callback_mask, \
			callback_function, \
			callback_arguments ):

		if self.debug:
			print "MpCa.PV.add_callback() for PV '%s'" % self.pvname
			print \
	"  callback_mask = %s, callback_function = %s, callback_arguments = %s"\
		    % ( callback_mask, callback_function, callback_arguments )

		callback = Callback( callback_mask, \
				callback_function, callback_arguments )

		callback.pv = self

		MpCaPrivate.pv_add_callback( self.handle, callback )

		if self.debug:
			print "  callback =", callback

		return callback

